import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.connection_status import ConnectionStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ConnectionUpdate")


@_attrs_define
class ConnectionUpdate:
    """Schema for updating a connection

    Attributes:
        status (Union[ConnectionStatus, None, Unset]):
        last_ping (Union[None, Unset, datetime.datetime]):
        disconnected_at (Union[None, Unset, datetime.datetime]):
    """

    status: Union[ConnectionStatus, None, Unset] = UNSET
    last_ping: Union[None, Unset, datetime.datetime] = UNSET
    disconnected_at: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status: Union[None, Unset, str]
        if isinstance(self.status, Unset):
            status = UNSET
        elif isinstance(self.status, ConnectionStatus):
            status = self.status.value
        else:
            status = self.status

        last_ping: Union[None, Unset, str]
        if isinstance(self.last_ping, Unset):
            last_ping = UNSET
        elif isinstance(self.last_ping, datetime.datetime):
            last_ping = self.last_ping.isoformat()
        else:
            last_ping = self.last_ping

        disconnected_at: Union[None, Unset, str]
        if isinstance(self.disconnected_at, Unset):
            disconnected_at = UNSET
        elif isinstance(self.disconnected_at, datetime.datetime):
            disconnected_at = self.disconnected_at.isoformat()
        else:
            disconnected_at = self.disconnected_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if last_ping is not UNSET:
            field_dict["last_ping"] = last_ping
        if disconnected_at is not UNSET:
            field_dict["disconnected_at"] = disconnected_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_status(data: object) -> Union[ConnectionStatus, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                status_type_0 = ConnectionStatus(data)

                return status_type_0
            except:  # noqa: E722
                pass
            return cast(Union[ConnectionStatus, None, Unset], data)

        status = _parse_status(d.pop("status", UNSET))

        def _parse_last_ping(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_ping_type_0 = isoparse(data)

                return last_ping_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        last_ping = _parse_last_ping(d.pop("last_ping", UNSET))

        def _parse_disconnected_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                disconnected_at_type_0 = isoparse(data)

                return disconnected_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        disconnected_at = _parse_disconnected_at(d.pop("disconnected_at", UNSET))

        connection_update = cls(
            status=status,
            last_ping=last_ping,
            disconnected_at=disconnected_at,
        )

        connection_update.additional_properties = d
        return connection_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
